function [ varargout ] = luRozklad( A )
% LU rozklad matice s stenou pivotac
%
%   [ Y, p ] = luRozklad( A )
%   Vstup:  A ... regulrn matice n x n
%   Vstup: Y ... matice obsahujc horn trojhelnkovou matici U a doln
%                 trojhelnkovou matici L bez jednotkov diagonly, 
%                 Y = U + L - eye(n)
%           p ... permutan vektor, A(p, :) = L * U
%   [ L, U, P ] = luRozklad( A )
%   Vstup: L ... doln trojhelnkov matice
%           U ... horn trojhelnkov matice
%           P ... permutan matice, P * A = L * U
%
%   Datum posledn pravy: 23.7.2015, KF

[n, n2] = size(A);
if n ~= n2
    error ('Zadan matice nen tvercov.')
end
% Inicializace permutanho vektoru
p = 1 : n;
for k = 1 : n - 1
    % Nalezen maxima ve sloupci k (sloupcov vbr pivota)
    [maximum, imax] = max(abs(A(k : n, k)));
    if maximum < eps
        error ('LU:singularniMatice', 'Zadan matice je singulrn.')
    end
    r = imax + k - 1;
    % Vmna dku k s dkem r v matici A a zaznamenn informace o vmn
    % do permutanho vektoru
    if r ~= k
        A([r k], :) = A([k r], :);
        p([r k]) = p([k r]);
    end
    % Eliminace poddiagonlnch prvk sloupce k
    for i = k + 1 : n
        A(i, k) = A(i, k) / A(k, k); % Vpoet multipliktoru
        A(i, k + 1 : n) = A(i, k + 1 : n) - A(i, k) * A(k, k + 1 : n);
    end
end
if abs(A(n, n)) < eps
    error ('LU:singularniMatice', 'Zadan matice je singulrn.')
end
if nargout == 2
    varargout{1} = A;
    varargout{2} = p;
end
if nargout == 3
    I = eye(n);
    varargout{1} = I + tril(A, -1); % Matice L
    varargout{2} = triu(A);         % Matice U
    varargout{3} = I(p, :);         % Matice P
end
end

