function [ A, baz ] = rrefTvar( A, tol )
% Vpoet redukovanho odstupovanho tvaru matice (RREF) 
% za vyuit Gauss-Jordanovy eliminace s stenou pivotac
%
%   [ R, baz ] = rrefTvar( A, tol )
%   Vstup:  A ... matice m x n
%           tol ... tolerance pro rozpoznn nuly
%   Vstup: R ... matice v redukovanm odstupovanm tvaru
%           baz ... vektor obsahujc indexy bzovch sloupc matice A
%
%   Datum posledn pravy: 8.8.2015, KF

[m, n] = size(A);
i = 1;
j = 1;
baz = zeros(1, n);
while i <= m && j <= n
    % Nalezen maxima ve sloupci j (sloupcov vbr pivota)
    [maximum, imax] = max(abs(A(i : m, j)));    
    if maximum <= tol
        % Prvky sloupce j jsou zanedbateln mal a jsou vynulovny
        A(i : m, j) = zeros(m - i + 1, 1);
        % Hledn pivota pokrauje v sousednm sloupci
        j = j + 1;    
    else        
        k = imax + i - 1;
        % Vmna i-tho a k-tho dku matice
        if i ~= k
            A([i k], j : n) = A([k i], j : n);
        end
        % Vydlen pivotnho dku hodnotou pivota
        A(i, j : n) = A(i, j : n) / A(i, j);
        % Eliminace prvk j-tho sloupce nachzejcch se
        % nad a pod pivotem
        for k = [1 : i - 1 i + 1 : m]   
            A(k, j + 1 : n) = A(k, j + 1 : n) - A(k, j) * A(i, j + 1 : n);
            A(k, j) = 0;
        end 
        % Sloupec j je bzovm sloupcem
        baz(i) = j;
    i = i + 1;
    j = j + 1;
    end
end
baz = baz(1 : i - 1);
end

