function [ varargout ] = qrRozklad( A )
% QR rozklad matice uitm Householderovch reflex
%
%   [ Y ] = qrRozklad( A )
%   Vstup:  A ... matice m x n
%   Vstup: Y ... matice, pro kterou R = triu(Y) a v jej poddiagonln 
%                 sti jsou uloeny pslun Householderovy vektory 
%                 (krom prvn sloky vektoru, kter je vdy rovna jedn)
%   [ Q, R ] = qrRozklad( A )
%   Vstup:  A ... matice m x n
%   Vstup: Q ... ortogonln matice m x m
%           R ... horn trojhelnkov matice m x n,
%                 A = Q * R
%
%   Datum posledn pravy: 1.11.2015, KF

[m, n] = size(A);
beta = zeros(1, min(n, m));
for j = 1 : min(n, m - 1)
    % Vpoet j-tho Householderova vektoru
    x = A(j : m, j);
    sigma = x(2 : end)' * x(2 : end);
    v = [1; x(2 : end)];
    if sigma == 0 && x(1) >= 0
        beta(j) = 0;
    elseif sigma == 0 && x(1) < 0
        beta(j) = 2;
    else
        mi = sqrt(sigma + x(1) ^ 2);
        if x(1) < 0
            v(1) = x(1) - mi;
        else
            v(1) = -sigma / (x(1) + mi);
        end
        beta(j) = 2 * v(1) ^ 2 / (sigma + v(1) ^ 2);
        v = v / v(1);
    end
    % Aktualizace matice A (A = Q_j * A)
    A(j : m, j : n) = A(j : m, j : n) - (beta(j) * v) * (v' * A(j : m, j : n));
    % Uloen Householderova vektoru do poddiagonln sti matice A
    A(j + 1 : m, j) = v(2 : end);
end
if nargout == 1
    varargout{1} = A;
end
if nargout == 2
    % Konstrukce matice Q
    Q = eye(m);
    for j = min(n, m - 1) : -1 : 1
        v = [1; A(j + 1 : m, j)];
        Q(j : m, j : m) = Q(j : m, j : m) - (beta(j) * v) * (v' * Q(j : m, j : m));
    end
    varargout{1} = Q;
    varargout{2} = triu(A); % Matice R
end
end

