function [ U, S, V ] = svdRozklad( A, tol )
% SVD rozklad matice vypoten pomoc spektrlnho rozkladu a QR rozkladu 
%
%   [ U, S, V ] = svdRozklad( A, tol )
%   Vstup:  A ... matice m x n
%           tol ... tolerance udvajc poadovanou pesnost vsledk
%   Vstup: U ... ortogonln matice m x m, jej sloupce jsou tvoeny 
%                 levmi singulrnmi vektory matice A
%           S ... diagonln matice m x n, jej diagonln prvky jsou
%                 singulrn sla matice A
%           V ... ortogonln matice n x n, jej sloupce jsou tvoeny 
%                 pravmi singulrnmi vektory matice A,
%                 A = U * S * V'
%
%   Datum posledn pravy: 1.11.2015, KF

[m, n] = size(A);
if m >= n
    [V, ~] = spektRozklad(A' * A, tol);
    [U, S] = qrRozklad(A * V);
    S = tril(S);
else
    [U, ~] = spektRozklad(A * A', tol);
    [V, S] = qrRozklad(A' * U);
    S = tril(S)';
end
end

